use std::convert::Infallible;

use bytes::Bytes;
use thiserror::Error;

mod v1;

pub use v1::Client;

#[derive(Debug, Error)]
pub enum MessagingError {
    #[error("failed to connect to messaging transport")]
    ConnectError,
    #[error("failed to publish message to messaging transport")]
    PublishError,
    #[error("failed to subscribe to messaging transport")]
    SubscribeError,
}

pub struct RawData(Bytes);

impl TryFrom<async_nats::Message> for RawData {
    type Error = Infallible;

    fn try_from(value: async_nats::Message) -> Result<Self, Self::Error> {
        Ok(Self(value.payload))
    }
}

pub async fn init_services(client: Client) -> anyhow::Result<()> {
    self::v1::terminal::init_terminal_service(client.clone()).await?;
    self::v1::exec::init_exec_service(client).await?;

    Ok(())
}
